from Acquisition import aq_parent, aq_inner
from urllib import urlencode

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFPlone.utils import safe_unicode
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from Products.Five.utilities import marker

from plone.app.layout.icons.interfaces import IContentIcon
from plone.app.layout.icons.icons import BaseIcon
from plone.i18n.normalizer.interfaces import IIDNormalizer

from plone.z3cform.layout import wrap_form

from zope.formlib import form
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.traversing.interfaces import ITraversable
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.i18nmessageid import MessageFactory
from zope.event import notify
_ = MessageFactory('plone')

from z3c.form import form as z3cform, field as z3cfield
from z3c.form import widget, button

from Products.ATContentTypes.interface import IATFolder, IATTopic

from Solgema.EnvironmentViewlets.config import _sbd
from Solgema.EnvironmentViewlets import event, interfaces, options

def view_url(context):
    """Last part of the url for viewing this context.

    By default: for Images and Files, redirect to .../view

    Code taken from CMFPlone/skins/plone_scripts/livesearch_reply.py
    """
    portalProperties = getToolByName(context, 'portal_properties')
    siteProperties = getattr(portalProperties, 'site_properties', None)
    useViewAction = []
    if siteProperties is not None:
        useViewAction = siteProperties.getProperty('typesUseViewActionInListings', [])
    extra = ''
    if context.portal_type in useViewAction:
        extra = '/view'
    return extra

class EnvironmentFormView( z3cform.EditForm ):

    adapters = None
    interface = None
    marker = None
    fields = z3cfield.Fields()

class BaseEnvironmentForm( EnvironmentFormView ):

    def allowed( self ):
        adapter = component.queryAdapter( self.context, self.interface )
        return not ( adapter is None )

class EnvironmentFormCreation( BaseEnvironmentForm ):

    actions = form.Actions()

    def update( self ):
        marker.mark( self.context, self.marker )
        self.adapters = { self.interface : options.PropertyBag.makeinstance( self.interface ),
                          self.interface : options.PropertyBag.makeinstance( self.interface ) }

        return super( BaseEnvironmentForm, self).update()

    @button.buttonAndHandler(_('label_enable'), name='activate')
    def activateEnvironment( self, action):
        data, errors = self.extractData()
        changes = self.applyChanges(data)

#        self.adapters[ interfaces.IRealEstateContent ].made_realestate_by = getSecurityManager().getUser().getId()
        notify(
            event.EnvironmentCreationEvent( self.context, self.adapters[ self.interface ], self.interface )
            )

        # redirect to view

        translated_message = self.context.utranslate(u'Changes saved.', domain='plone').encode(self.context.getCharset())
        encoded_message = urlencode({'portal_status_message' : translated_message})
        extra = view_url(self.context)
        self.request.response.redirect( '%s%s?%s' % (self.context.absolute_url(), extra, encoded_message) )

    @button.buttonAndHandler(_('label_cancel'), name='cancel')
    def handleCancel( self, action):
        marker.erase( self.context, self.marker )
        self.request.RESPONSE.redirect( self.context.absolute_url() )

class EnvironmentEdit( BaseEnvironmentForm ):

    @button.buttonAndHandler(_('label_save'), name='apply')
    def handleApply(self, action):
        data, errors = self.extractData()
        changes = self.applyChanges(data)
        if changes:
            notify(
                event.EnvironmentModificationEvent( self.context, self.interface, self.interface )
                )
            self.status = self.successMessage
        else:
            self.status = self.noChangesMessage
        extra = view_url(self.context)
        self.request.response.redirect( '%s%s' % (self.context.absolute_url(), extra) )

    @button.buttonAndHandler(_('label_cancel'), name='cancel')
    def handleCancel( self, action):
        self.request.RESPONSE.redirect( self.context.absolute_url() )

##### BANNER

class SolgemaBandeauDestruction( BrowserView ):
    marker = interfaces.IBandeauMarker
    def __call__(self):
        marker.erase( self.context, self.marker )
        self.context.reindexObject()
        extra = view_url(self.context)
        self.request.response.redirect( '%s%s' % (self.context.absolute_url(), extra) )

class BandeauForm( BaseEnvironmentForm ):
    interface = interfaces.IBandeauContent
    marker = interfaces.IBandeauMarker
    fields = z3cfield.Fields( interface )

class ZBandeauCreation( BandeauForm, EnvironmentFormCreation ):
    actions = EnvironmentFormCreation.actions
    update  = EnvironmentFormCreation.update

class ZBandeauEdit( BandeauForm, EnvironmentEdit ):
    pass

SolgemaBandeauCreation = wrap_form(ZBandeauCreation)
SolgemaBandeauEdit = wrap_form(ZBandeauEdit)

##### FOOTER

class FooterForm( BaseEnvironmentForm ):
    interface = interfaces.IFooterContent
    marker = interfaces.IFooterMarker
    fields = z3cfield.Fields( interface )

class ZFooterCreation(FooterForm, EnvironmentFormCreation ):
    actions = EnvironmentFormCreation.actions
    update  = EnvironmentFormCreation.update

class ZFooterEdit(FooterForm, EnvironmentEdit ):
    pass

SolgemaFooterCreation = wrap_form(ZFooterCreation)
SolgemaFooterEdit = wrap_form(ZFooterEdit)

class SolgemaFooterDestruction( SolgemaBandeauDestruction ):
    marker = interfaces.IFooterMarker

##### PRINT FOOTER

class PrintFooterForm( BaseEnvironmentForm ):
    interface = interfaces.IPrintFooterContent
    marker = interfaces.IPrintFooterMarker
    fields = z3cfield.Fields( interface )

class ZPrintFooterCreation(PrintFooterForm, EnvironmentFormCreation ):
    actions = EnvironmentFormCreation.actions
    update  = EnvironmentFormCreation.update

class ZPrintFooterEdit(PrintFooterForm, EnvironmentEdit ):
    pass

SolgemaPrintFooterCreation = wrap_form(ZPrintFooterCreation)
SolgemaPrintFooterEdit = wrap_form(ZPrintFooterEdit)

class SolgemaPrintFooterDestruction( SolgemaBandeauDestruction ):
    marker = interfaces.IPrintFooterMarker

##### LOGO

class LogoForm( BaseEnvironmentForm ):
    interface = interfaces.ILogoContent
    marker = interfaces.ILogoMarker
    fields = z3cfield.Fields( interface )

class ZLogoCreation(LogoForm, EnvironmentFormCreation ):
    actions = EnvironmentFormCreation.actions
    update  = EnvironmentFormCreation.update

class ZLogoEdit(LogoForm, EnvironmentEdit ):
    pass

SolgemaLogoCreation = wrap_form(ZLogoCreation)
SolgemaLogoEdit = wrap_form(ZLogoEdit)

class SolgemaLogoDestruction( SolgemaBandeauDestruction ):
    marker = interfaces.ILogoMarker

##### PRINT LOGO

class PrintLogoForm( BaseEnvironmentForm ):
    interface = interfaces.IPrintLogoContent
    marker = interfaces.IPrintLogoMarker
    fields = z3cfield.Fields( interface )

class ZPrintLogoCreation(PrintLogoForm, EnvironmentFormCreation ):
    actions = EnvironmentFormCreation.actions
    update  = EnvironmentFormCreation.update

class ZPrintLogoEdit(PrintLogoForm, EnvironmentEdit ):
    pass

SolgemaPrintLogoCreation = wrap_form(ZPrintLogoCreation)
SolgemaPrintLogoEdit = wrap_form(ZPrintLogoEdit)

class SolgemaPrintLogoDestruction( SolgemaBandeauDestruction ):
    marker = interfaces.IPrintLogoMarker


class SolgemaBandeauContentControl( BrowserView ):
    """ conditions for presenting various actions
    """

    __allow_access_to_unprotected_subobjects__ = 1
    #__slots__ = ( 'context', 'request', 'options' )

    def __init__( self, context, request ):
        self.context = context
        self.request = request

        portal = getToolByName( self.context, 'portal_url').getPortalObject()

    def canBandeau( self ):
        if hasattr(self.context, 'getText'):
            return True
        if hasattr(self.context, 'tag'):
            return True
        if self.context.portal_type == 'Collage':
            return True
        if self.context.portal_type == 'FlashMovie':
            return True
        return False

    def canLogo( self ):
        if hasattr(self.context, 'getText'):
            return True
        if hasattr(self.context, 'tag'):
            return True
        if self.context.portal_type == 'FlashMovie':
            return True
        return False

    # BANNER

    def isBandeau( self ):
        """  does the context implement the IRealEstate interface
        """
        return interfaces.IBandeauMarker.providedBy( self.context )

    isBandeau.__roles__ = None

    def allowActivateBandeau( self ):
        return self.canBandeau() and not self.isBandeau()

    allowActivateBandeau.__roles__ = None

    def allowDeactivateBandeau( self ):
        return self.isBandeau()

    allowDeactivateBandeau.__roles__ = None

    # FOOTER

    def isFooter( self ):
        """  does the context implement the IRealEstate interface
        """
        return interfaces.IFooterMarker.providedBy( self.context )

    isFooter.__roles__ = None

    def allowActivateFooter( self ):
        return self.canBandeau() and not self.isFooter()

    allowActivateFooter.__roles__ = None

    def allowDeactivateFooter( self ):
        return self.isFooter()

    allowDeactivateFooter.__roles__ = None

    # Print FOOTER

    def isPrintFooter( self ):
        """  does the context implement the IRealEstate interface
        """
        return interfaces.IPrintFooterMarker.providedBy( self.context )

    isPrintFooter.__roles__ = None

    def allowActivatePrintFooter( self ):
        return self.canBandeau() and not self.isPrintFooter()

    allowActivatePrintFooter.__roles__ = None

    def allowDeactivatePrintFooter( self ):
        return self.isPrintFooter()

    allowDeactivatePrintFooter.__roles__ = None

    # Logo

    def isLogo( self ):
        """  does the context implement the IRealEstate interface
        """
        return interfaces.ILogoMarker.providedBy( self.context )

    isLogo.__roles__ = None

    def allowActivateLogo( self ):
        return self.canLogo() and not self.isLogo()

    allowActivateLogo.__roles__ = None

    def allowDeactivateLogo( self ):
        return self.isLogo()

    allowDeactivateLogo.__roles__ = None

    # Print Logo

    def isPrintLogo( self ):
        """  does the context implement the IRealEstate interface
        """
        return interfaces.IPrintLogoMarker.providedBy( self.context )

    isPrintLogo.__roles__ = None

    def allowActivatePrintLogo( self ):
        return self.canLogo() and not self.isPrintLogo()

    allowActivatePrintLogo.__roles__ = None

    def allowDeactivatePrintLogo( self ):
        return self.isPrintLogo()

    allowDeactivatePrintLogo.__roles__ = None

